use crate::{amount, asset::AssetId, storage, transaction, AccountId};
use serde::{Serialize, Serializer};
use std::fmt::Display;

#[derive(thiserror::Error, Debug, Serialize)]
pub enum Error {
    #[error("Transaction: {0}")]
    Transaction(#[from] transaction::Error),

    #[error("Storage: {0}")]
    Storage(#[from] storage::Error),

    #[error("Asset {0} is not defined")]
    AssetIdNotFound(AssetId),

    #[error("Asset {0} is not defined")]
    AssetNotFound(String),

    #[error("Not enough funds (asset {1}) for account {0}")]
    InsufficientBalance(AccountId, AssetId),

    #[error("Invalid amount: {0}")]
    InvalidAmount(#[from] amount::Error),
}

pub fn serialize_to_string<S, T>(value: &T, serializer: S) -> Result<S::Ok, S::Error>
where
    S: Serializer,
    T: ToString + Display,
{
    serializer.serialize_str(&value.to_string())
}
